@@ -1,9 +1,10 @@
 from __future__ import annotations
 
-__version__ = "1.7.1"
+__version__ = "1.8.0"
 
 
 import asyncio
+import contextlib
 import inspect
 import logging
 import platform
@@ -16,6 +17,14 @@ from bleak.backends.device import BLEDevice
 from bleak.backends.service import BleakGATTServiceCollection
 
 CAN_CACHE_SERVICES = platform.system() == "Linux"
+
+if CAN_CACHE_SERVICES:
+    with contextlib.suppress(ImportError):  # pragma: no cover
+        from bleak.backends.bluezdbus import defs  # pragma: no cover
+        from bleak.backends.bluezdbus.manager import (  # pragma: no cover
+            get_global_bluez_manager,
+        )
+
 BLEAK_HAS_SERVICE_CACHE_SUPPORT = (
     "dangerous_use_bleak_cache" in inspect.signature(BleakClient.connect).parameters
 )
@@ -87,6 +96,15 @@ class BleakClientWithServiceCache(BleakClient):
         super().__init__(*args, **kwargs)
         self._cached_services: BleakGATTServiceCollection | None = None
 
+    @property
+    def _has_service_cache(self) -> bool:
+        """Check if we can cache services and there is a cache."""
+        return (
+            not BLEAK_HAS_SERVICE_CACHE_SUPPORT
+            and CAN_CACHE_SERVICES
+            and self._cached_services is not None
+        )
+
     async def connect(
         self, *args: Any, dangerous_use_bleak_cache: bool = False, **kwargs: Any
     ) -> bool:
@@ -96,26 +114,28 @@ class BleakClientWithServiceCache(BleakClient):
             Boolean representing connection status.
 
         """
+        if self._has_service_cache and await self._services_vanished():
+            _LOGGER.debug("Clear cached services since they have vanished")
+            self._cached_services = None
+
         connected = await super().connect(
             *args, dangerous_use_bleak_cache=dangerous_use_bleak_cache, **kwargs
         )
+
         if (
             connected
             and not dangerous_use_bleak_cache
             and not BLEAK_HAS_SERVICE_CACHE_SUPPORT
         ):
             self.set_cached_services(self.services)
+
         return connected
 
     async def get_services(
         self, *args: Any, dangerous_use_bleak_cache: bool = False, **kwargs: Any
     ) -> BleakGATTServiceCollection:
         """Get the services."""
-        if (
-            not BLEAK_HAS_SERVICE_CACHE_SUPPORT
-            and CAN_CACHE_SERVICES
-            and self._cached_services
-        ):
+        if self._has_service_cache:
             _LOGGER.debug("Cached services found: %s", self._cached_services)
             self.services = self._cached_services
             self._services_resolved = True
@@ -124,6 +144,19 @@ class BleakClientWithServiceCache(BleakClient):
             *args, dangerous_use_bleak_cache=dangerous_use_bleak_cache, **kwargs
         )
 
+    async def _services_vanished(self) -> bool:
+        """Check if the services have vanished."""
+        with contextlib.suppress(Exception):
+            device_path = self._device_path
+            manager = await get_global_bluez_manager()
+            for service_path, service_ifaces in manager._properties.items():
+                if (
+                    service_path.startswith(device_path)
+                    and defs.GATT_SERVICE_INTERFACE in service_ifaces
+                ):
+                    return False
+        return True
+
     def set_cached_services(self, services: BleakGATTServiceCollection | None) -> None:
         """Set the cached services."""
         self._cached_services = services
@@ -187,7 +220,9 @@ async def establish_connection(
 
     while True:
         attempt += 1
-        _LOGGER.debug("%s: Connecting (attempt: %s)", name, attempt)
+        _LOGGER.debug(
+            "%s: Connecting (attempt: %s, last rssi: %s)", name, attempt, device.rssi
+        )
         try:
             async with async_timeout.timeout(BLEAK_SAFETY_TIMEOUT):
                 await client.connect(
@@ -197,7 +232,10 @@ async def establish_connection(
         except asyncio.TimeoutError as exc:
             timeouts += 1
             _LOGGER.debug(
-                "%s: Timed out trying to connect (attempt: %s)", name, attempt
+                "%s: Timed out trying to connect (attempt: %s, last rssi: %s)",
+                name,
+                attempt,
+                device.rssi,
             )
             _raise_if_needed(name, exc)
         except BrokenPipeError as exc:
@@ -213,7 +251,11 @@ async def establish_connection(
             # BrokenPipeError: [Errno 32] Broken pipe
             transient_errors += 1
             _LOGGER.debug(
-                "%s: Failed to connect: %s (attempt: %s)", name, str(exc), attempt
+                "%s: Failed to connect: %s (attempt: %s, last rssi: %s)",
+                name,
+                str(exc),
+                attempt,
+                device.rssi,
             )
             _raise_if_needed(name, exc)
         except BLEAK_EXCEPTIONS as exc:
@@ -223,11 +265,17 @@ async def establish_connection(
             else:
                 connect_errors += 1
             _LOGGER.debug(
-                "%s: Failed to connect: %s (attempt: %s)", name, str(exc), attempt
+                "%s: Failed to connect: %s (attempt: %s, last rssi: %s)",
+                name,
+                str(exc),
+                attempt,
+                device.rssi,
             )
             _raise_if_needed(name, exc)
         else:
-            _LOGGER.debug("%s: Connected (attempt: %s)", name, attempt)
+            _LOGGER.debug(
+                "%s: Connected (attempt: %s, last rssi: %s)", name, attempt, device.rssi
+            )
             return client
         # Ensure the disconnect callback
         # has a chance to run before we try to reconnect
